@@ -121,10 +121,9 @@ static int action_open_plain(void)
 		.size = opt_size,
 	};
 	char *password = NULL;
-	size_t passwordLen;
+	size_t passwordLen, key_size_max;
 	size_t key_size = (opt_key_size ?: DEFAULT_PLAIN_KEYBITS) / 8;
 	uint32_t activate_flags = 0;
-	int keyfile_limited = 0;
 	int r;
 
 	r = crypt_parse_name_and_mode(opt_cipher ?: DEFAULT_CIPHER(PLAIN),
@@ -134,11 +133,8 @@ static int action_open_plain(void)
 		goto out;
 	}
 
-	if (opt_key_file && strcmp(opt_key_file, "-") != 0)
-		keyfile_limited = 1;
-
 	/* FIXME: temporary hack, no hashing for keyfiles in plain mode */
-	if (opt_key_file && keyfile_limited) {
+	if (opt_key_file && !tools_is_stdin(opt_key_file)) {
 		params.hash = NULL;
 		if (!opt_batch_mode && opt_hash)
 			log_std(_("WARNING: The --hash parameter is being ignored "
@@ -148,7 +144,7 @@ static int action_open_plain(void)
 	if (params.hash && !strcmp(params.hash, "plain"))
 		params.hash = NULL;
 
-	if (!opt_batch_mode && !params.hash && opt_key_file && keyfile_limited && opt_keyfile_size)
+	if (!opt_batch_mode && !params.hash && opt_key_file && !tools_is_stdin(opt_key_file) && opt_keyfile_size)
 		log_std(_("WARNING: The --keyfile-size option is being ignored, "
 			 "the read size is the same as the encryption key size.\n"));
 
@@ -172,23 +168,23 @@ static int action_open_plain(void)
 
 	_set_activation_flags(&activate_flags);
 
-	if (opt_key_file) {
+	if (!tools_is_stdin(opt_key_file)) {
 		/* If no hash, key is read directly, read size is always key_size
 		 * (possible opt_keyfile_size is ignored.
 		 * If hash is specified, opt_keyfile_size is applied.
 		 * The opt_keyfile_offset is applied always.
 		 */
+		key_size_max = params.hash ? (size_t)opt_keyfile_size : key_size;
 		r = crypt_activate_by_keyfile_offset(cd, action_argv[1],
-			CRYPT_ANY_SLOT, opt_key_file,
-			params.hash ? opt_keyfile_size : key_size, opt_keyfile_offset,
-			activate_flags);
+			CRYPT_ANY_SLOT, opt_key_file, key_size_max,
+			opt_keyfile_offset, activate_flags);
 	} else {
+		key_size_max = (opt_key_file && !params.hash) ? key_size : (size_t)opt_keyfile_size;
 		r = tools_get_key(_("Enter passphrase: "),
 				  &password, &passwordLen,
-				  opt_keyfile_offset, opt_keyfile_size,
-				  NULL, opt_timeout,
-				  _verify_passphrase(0), 0,
-				  cd);
+				  opt_keyfile_offset, key_size_max,
+				  opt_key_file, opt_timeout,
+				  _verify_passphrase(0), 0, cd);
 		if (r < 0)
 			goto out;
 
@@ -495,8 +491,8 @@ static int action_benchmark_kdf(const char *hash)
 	if (r < 0)
 		log_std("PBKDF2-%-9s     N/A\n", hash);
 	else
-		log_std("PBKDF2-%-9s %7" PRIu64 " iterations per second\n",
-			hash, kdf_iters);
+		log_std("PBKDF2-%-9s %7" PRIu64 " iterations per second for %d-bit key\n",
+			hash, kdf_iters, DEFAULT_LUKS1_KEYBITS);
 	return r;
 }
 
@@ -712,6 +708,10 @@ static int action_luksFormat(void)
 		goto out;
 	}
 
+	/* Never call pwquality if using null cipher */
+	if (tools_is_cipher_null(cipher))
+		opt_force_password = 1;
+
 	if ((r = crypt_init(&cd, header_device))) {
 		if (opt_header_device)
 			log_err(_("Cannot use %s as on-disk header.\n"), header_device);
@@ -968,6 +968,10 @@ static int action_luksAddKey(void)
 	if ((r = crypt_load(cd, CRYPT_LUKS1, NULL)))
 		goto out;
 
+	/* Never call pwquality if using null cipher */
+	if (tools_is_cipher_null(crypt_get_cipher(cd)))
+		opt_force_password = 1;
+
 	keysize = crypt_get_volume_key_size(cd);
 	/* FIXME: lib cannot properly set verification for new/old passphrase */
 	crypt_set_password_verify(cd, _verify_passphrase(0));
@@ -995,13 +999,15 @@ static int action_luksAddKey(void)
 
 		r = crypt_keyslot_add_by_volume_key(cd, opt_key_slot, key, keysize,
 						    password_new, password_new_size);
-	} else if (opt_key_file || opt_new_key_file) {
+	} else if (opt_key_file && !tools_is_stdin(opt_key_file) &&
+		   opt_new_key_file && !tools_is_stdin(opt_new_key_file)) {
 		r = crypt_keyslot_add_by_keyfile_offset(cd, opt_key_slot,
 			opt_key_file, opt_keyfile_size, opt_keyfile_offset,
 			opt_new_key_file, opt_new_keyfile_size, opt_new_keyfile_offset);
 	} else {
 		r = tools_get_key(_("Enter any existing passphrase: "),
-			      &password, &password_size, 0, 0, NULL,
+			      &password, &password_size,
+			      opt_keyfile_offset, opt_keyfile_size, opt_key_file,
 			      opt_timeout, _verify_passphrase(0), 0, cd);
 
 		if (r < 0)
@@ -1015,8 +1021,9 @@ static int action_luksAddKey(void)
 			goto out;
 
 		r = tools_get_key(_("Enter new passphrase for key slot: "),
-				  &password_new, &password_new_size, 0, 0, NULL,
-				  opt_timeout, _verify_passphrase(1), 1, cd);
+				  &password_new, &password_new_size,
+				  opt_new_keyfile_offset, opt_new_keyfile_size, opt_new_key_file,
+				  opt_timeout, _verify_passphrase(1), opt_new_key_file ? 0 : 1, cd);
 		if (r < 0)
 			goto out;
 
@@ -1046,6 +1053,10 @@ static int action_luksChangeKey(void)
 	if ((r = crypt_load(cd, CRYPT_LUKS1, NULL)))
 		goto out;
 
+	/* Never call pwquality if using null cipher */
+	if (tools_is_cipher_null(crypt_get_cipher(cd)))
+		opt_force_password = 1;
+
 	if (opt_iteration_time)
 		crypt_set_iteration_time(cd, opt_iteration_time);
 
@@ -1568,9 +1579,6 @@ int main(int argc, const char **argv)
 		usage(popt_context, EXIT_FAILURE, poptStrerror(r),
 		      poptBadOption(popt_context, POPT_BADOPTION_NOALIAS));
 
-	if (crypt_fips_mode())
-		crypt_log(NULL, CRYPT_LOG_VERBOSE, _("Running in FIPS mode.\n"));
-
 	if (opt_version_mode) {
 		log_std("%s %s\n", PACKAGE_NAME, PACKAGE_VERSION);
 		poptFreeContext(popt_context);
